@@ -9,6 +9,27 @@
 #' @seealso [Boost Documentation](https://www.boost.org/doc/libs/latest/libs/math/doc/html/math_toolkit/dist_ref/dists/extreme_dist.html) for more details on the mathematical background.
 #' @examples
 #' # Extreme Value distribution with location = 0, scale = 1
+#' dist <- extreme_value_distribution(0, 1)
+#' # Apply generic functions
+#' cdf(dist, 0.5)
+#' logcdf(dist, 0.5)
+#' pdf(dist, 0.5)
+#' logpdf(dist, 0.5)
+#' hazard(dist, 0.5)
+#' chf(dist, 0.5)
+#' mean(dist)
+#' median(dist)
+#' mode(dist)
+#' range(dist)
+#' quantile(dist, 0.2)
+#' standard_deviation(dist)
+#' support(dist)
+#' variance(dist)
+#' skewness(dist)
+#' kurtosis(dist)
+#' kurtosis_excess(dist)
+#'
+#' # Convenience functions
 #' extreme_value_pdf(0)
 #' extreme_value_lpdf(0)
 #' extreme_value_cdf(0)
@@ -16,32 +37,45 @@
 #' extreme_value_quantile(0.5)
 NULL
 
+#' @rdname extreme_value_distribution
+#' @export
+extreme_value_distribution <- function(location = 0, scale = 1) {
+  structure(
+    list(
+      extptr = .Call(`extreme_value_init_`, location, scale),
+      location = location,
+      scale = scale
+    ),
+    class = c("extreme_value_distribution", "boost_distribution")
+  )
+}
+
 #' @rdname extreme_value_distribution
 #' @export
 extreme_value_pdf <- function(x, location = 0, scale = 1) {
-  .Call(`extreme_value_pdf_`, x, location, scale)
+  pdf(extreme_value_distribution(location, scale), x)
 }
 
 #' @rdname extreme_value_distribution
 #' @export
 extreme_value_lpdf <- function(x, location = 0, scale = 1) {
-  .Call(`extreme_value_logpdf_`, x, location, scale)
+  logpdf(extreme_value_distribution(location, scale), x)
 }
 
 #' @rdname extreme_value_distribution
 #' @export
 extreme_value_cdf <- function(x, location = 0, scale = 1) {
-  .Call(`extreme_value_cdf_`, x, location, scale)
+  cdf(extreme_value_distribution(location, scale), x)
 }
 
 #' @rdname extreme_value_distribution
 #' @export
 extreme_value_lcdf <- function(x, location = 0, scale = 1) {
-  .Call(`extreme_value_logcdf_`, x, location, scale)
+  logcdf(extreme_value_distribution(location, scale), x)
 }
 
 #' @rdname extreme_value_distribution
 #' @export
 extreme_value_quantile <- function(p, location = 0, scale = 1) {
-  .Call(`extreme_value_quantile_`, p, location, scale)
+  quantile(extreme_value_distribution(location, scale), p)
 }
