@@ -11,6 +11,24 @@
 #' @examples
 #' # Noncentral Beta distribution with shape parameters alpha = 2, beta = 3
 #' # and noncentrality parameter lambda = 1
+#' dist <- non_central_beta_distribution(2, 3, 1)
+#' # Apply generic functions
+#' cdf(dist, 0.5)
+#' logcdf(dist, 0.5)
+#' pdf(dist, 0.5)
+#' logpdf(dist, 0.5)
+#' hazard(dist, 0.5)
+#' chf(dist, 0.5)
+#' mean(dist)
+#' median(dist)
+#' mode(dist)
+#' range(dist)
+#' quantile(dist, 0.2)
+#' standard_deviation(dist)
+#' support(dist)
+#' variance(dist)
+#'
+#' # Convenience functions
 #' non_central_beta_pdf(0.5, 2, 3, 1)
 #' non_central_beta_lpdf(0.5, 2, 3, 1)
 #' non_central_beta_cdf(0.5, 2, 3, 1)
@@ -18,32 +36,46 @@
 #' non_central_beta_quantile(0.5, 2, 3, 1)
 NULL
 
+#' @rdname non_central_beta_distribution
+#' @export
+non_central_beta_distribution <- function(alpha, beta, lambda) {
+  structure(
+    list(
+      extptr = .Call(`non_central_beta_init_`, alpha, beta, lambda),
+      alpha = alpha,
+      beta = beta,
+      non_centrality = lambda
+    ),
+    class = c("non_central_beta_distribution", "boost_distribution")
+  )
+}
+
 #' @rdname non_central_beta_distribution
 #' @export
 non_central_beta_pdf <- function(x, alpha, beta, lambda) {
-  .Call(`non_central_beta_pdf_`, x, alpha, beta, lambda)
+  pdf(non_central_beta_distribution(alpha, beta, lambda), x)
 }
 
 #' @rdname non_central_beta_distribution
 #' @export
 non_central_beta_lpdf <- function(x, alpha, beta, lambda) {
-  .Call(`non_central_beta_logpdf_`, x, alpha, beta, lambda)
+  logpdf(non_central_beta_distribution(alpha, beta, lambda), x)
 }
 
 #' @rdname non_central_beta_distribution
 #' @export
 non_central_beta_cdf <- function(x, alpha, beta, lambda) {
-  .Call(`non_central_beta_cdf_`, x, alpha, beta, lambda)
+  cdf(non_central_beta_distribution(alpha, beta, lambda), x)
 }
 
 #' @rdname non_central_beta_distribution
 #' @export
 non_central_beta_lcdf <- function(x, alpha, beta, lambda) {
-  .Call(`non_central_beta_logcdf_`, x, alpha, beta, lambda)
+  logcdf(non_central_beta_distribution(alpha, beta, lambda), x)
 }
 
 #' @rdname non_central_beta_distribution
 #' @export
 non_central_beta_quantile <- function(p, alpha, beta, lambda) {
-  .Call(`non_central_beta_quantile_`, p, alpha, beta, lambda)
+  quantile(non_central_beta_distribution(alpha, beta, lambda), p)
 }
