@@ -1,5 +1,4 @@
 #include "rootfinding_and_minimisation/without_derivatives.cpp"
 #include "rootfinding_and_minimisation/with_derivatives.cpp"
-#include "rootfinding_and_minimisation/cubic_roots.cpp"
-#include "rootfinding_and_minimisation/quartic_roots.cpp"
+#include "rootfinding_and_minimisation/polynomial_roots.cpp"
 #include "rootfinding_and_minimisation/minima.cpp"
